@@ -13,32 +13,25 @@ class LegacyApplication(BaseAuthBackend):
 
 
     def __init__(self, client_id = None, client_secret=None, username=None, password=None, host="localhost", port= 8000, protocol = "http", scopes= ["read"], parent=None, **kwargs) -> None:
-        self.client_id = client_id
-        self.client_secret = client_secret
-        assert self.client_id is not None, "Please provide a client_id argument"
-        assert self.client_secret is not None, "Please provide a client_secret argument"
+        assert client_id is not None, "Please provide a client_id argument"
+        assert client_secret is not None, "Please provide a client_secret argument"
+        super().__init__(host, port, protocol=protocol)
+
         self.username = username
         self.password = password
-
-        # Generate correct URLs
-        self.base_url = f"{protocol}://{host}:{port}/o/"
-        self.auth_url = self.base_url + "authorize"
-        self.token_url = self.base_url + "token"
+        self.client_id = client_id
+        self.client_secret = client_secret
+        self.auth_url = self.base_url + "o/authorize"
+        self.token_url = self.base_url + "o/token"
 
         # If you want to have a hosting QtWidget
         self.parent = parent
-
-        self.token = None
-
         self.legacy_app_client =  LegacyApplicationClient(self.client_id)
-
         self.scopes = scopes
 
-        super().__init__()
-
-    def getToken(self, loop=None) -> str:
-        if self.token: return self.token
+        
 
+    def fetchToken(self, loop=None) -> str:
         # Getting token
         if not self.username: self.username = input("Enter your username:    ")
         if not self.password: self.password = input("Password?               ")
@@ -53,15 +46,12 @@ class LegacyApplication(BaseAuthBackend):
 
 
         if "access_token" in response:
-            self.token = response["access_token"]
-            return self.token
+            return response["access_token"]
 
         else:
             raise Exception(f"Wasn't authorized! {response}")
 
 
-
-
     def getClientType(self):
         return ClientType.EXTERNAL
 
