@@ -11,7 +11,13 @@ class AssignationRequestMetaAuthModel(MessageMetaExtensionsModel):
     token: str
 
 class AssignationRequestMetaExtensionsModel(MessageMetaExtensionsModel):
+    # Set by postman consumer
     progress: Optional[str]
+    callback: Optional[str]
+
+    # Set by postman
+    with_progress: Optional[bool]  = False
+    with_callback: Optional[bool] = False
 
 class AssignationRequestMetaModel(MessageMetaModel):
     type: str = ASSIGNATION_REQUEST
