@@ -1,6 +1,7 @@
 
 
 from abc import ABC, abstractmethod
+from asyncio.events import AbstractEventLoop
 from typing import Union
 from bergen.utils import ExpansionError, expandInputs, shrinkOutputs
 from bergen.messages.assignation import AssignationMessage
@@ -14,7 +15,7 @@ from bergen.models import Node, Pod
 import inspect
 import sys
 from aiostream import stream
-import concurrent
+from concurrent.futures import ThreadPoolExecutor, ProcessPoolExecutor
 from functools import partial
 logger = logging.getLogger()
 
@@ -32,7 +33,7 @@ class BaseHelper(ABC):
         pass
 
     @abstractmethod
-    async def pass_progress(self, message, value):
+    async def pass_progress(self, message, value, percentage=None):
         pass
 
     @abstractmethod
@@ -46,14 +47,35 @@ class BaseHelper(ABC):
 
 class AssignationHelper():
 
-    def __init__(self, peasent_helper: BaseHelper, assignation) -> None:
+    def __init__(self, peasent_helper: BaseHelper, message: AssignationMessage, loop: AbstractEventLoop = None) -> None:
         self.peasent_helper = peasent_helper
-        self.assignation = assignation
+        self.message = message
+        self.loop = loop
+        pass
+
+    @abstractmethod
+    def progress(self, value, percentage=None):
         pass
 
 
-    def progress(self, value):
-        self.peasent_helper.pass_progress(self.message, value)
+
+class ThreadedAssignationHelper(AssignationHelper):
+
+    def progress(self, value, percentage=None):
+        logger.info(f'{percentage if percentage else "--"} : {value}')
+        if self.loop.is_running():
+            future = asyncio.run_coroutine_threadsafe(self.peasent_helper.pass_progress(self.message, value, percentage=percentage), self.loop)
+            return future.result()
+        else:
+            self.loop.run_until_complete(self.peasent_helper.pass_progress(self.message, value, percentage=percentage))
+
+
+class AsyncAssignationHelper(AssignationHelper):
+
+    async def progress(self, value, percentage=None):
+        logger.info(f'{percentage if percentage else "--"} : {value}')
+        await self.peasent_helper.pass_progress(self.message, value, percentage=percentage)
+
 
 
 async def shrink(node_template_pod, outputs):
@@ -98,7 +120,7 @@ class BasePeasent(ABC):
     helperClass = None
 
 
-    def __init__(self, *args, loop = None, name = None, **kwargs) -> None:
+    def __init__(self, *args, loop = None, name = None, raise_exceptions_local=False, **kwargs) -> None:
         
         
         self.unique_name = name or namegenerator.gen()
@@ -106,12 +128,17 @@ class BasePeasent(ABC):
         self.pods = {}
 
 
+        self.raise_exceptions_local = raise_exceptions_local
+
+
         self.nodes_to_register = [] # for Nodes to register
         self.template_function_set = [] # for Templates to register
         self.pod_function_set = [] # For Pods to register
 
         self.peasent_helper: BaseHelper = self.helperClass(self)
-    
+
+        self.threadpool = ThreadPoolExecutor(5)
+        self.processpool = ProcessPoolExecutor(5)
 
         self.podid_function_map = {}
         self.podid_pod_map = {}
@@ -132,12 +159,12 @@ class BasePeasent(ABC):
 
             async def wrapper(message: AssignationMessage):
                 
-                assignation_helper = AssignationHelper(self.peasent_helper, message)
-                kwargs = await expand(node_template_pod,message.data.inputs)
-
+                
                 try:
+                    kwargs = await expand(node_template_pod,message.data.inputs)
 
                     if is_coroutine:
+                        assignation_helper = AsyncAssignationHelper(self.peasent_helper, message, loop=self.loop)
                         result = await function(assignation_helper, **kwargs)
 
                         outputs = await shrink(node_template_pod, result)
@@ -145,6 +172,7 @@ class BasePeasent(ABC):
 
                         
                     elif is_asyncgen:
+                        assignation_helper = AsyncAssignationHelper(self.peasent_helper, message, loop=self.loop)
                         yieldstream = stream.iterate(function(assignation_helper, **kwargs))
                         lastresult = None
                         async with yieldstream.stream() as streamer:
@@ -155,12 +183,9 @@ class BasePeasent(ABC):
                         await self.peasent_helper.pass_result(message, lastresult)
 
                     elif is_function:  
-                        if threadhelper:
-                            result = await threadhelper(function, assignation_helper, **kwargs)
-                        else:
-                            result = function(assignation_helper, **kwargs)
-                            #helper.end(result)
-                        #result = await self.loop.run_in_executor(self.executor, partial(function,**kwargs), assignation_helper)
+                        assignation_helper = ThreadedAssignationHelper(self.peasent_helper, message, loop=self.loop)
+                        result = await self.loop.run_in_executor(self.threadpool, partial(function,**kwargs), assignation_helper)
+
                         outputs = await shrink(node_template_pod, result)
                         await self.peasent_helper.pass_result(message, outputs)
 
@@ -169,6 +194,8 @@ class BasePeasent(ABC):
 
                 except Exception as e:
                     await self.peasent_helper.pass_exception(message, e)
+                    logger.error(e)
+                    if self.raise_exceptions_local: raise e
                 
                 
                 
