@@ -62,7 +62,7 @@ class ListQuery(list):
 
     def to_df(self, exclude_unset=True, **kwargs):
         import pandas as pd
-        if not self._df: self._df = pd.DataFrame.from_records([rec.dict(exclude_unset=exclude_unset, **kwargs) for rec in self])
+        if not self._df: self._df = pd.json_normalize([rec.dict(exclude_unset=exclude_unset, **kwargs) for rec in self])
         return self._df
 
     def _repr_html_(self) -> str:
